<?php
/**
 * Created by Code Monkeys LLC
 * http://www.codemonkeysllc.com
 * User: Spencer
 * Date: 11/27/2017
 * Time: 4:33 PM
 *
 * THIS PLUGIN IS SIMPLY AN INTERFACE TO THE HIPAA FORMS SERVICE.  THE ONLY THINGS FROM THIS PLUGIN THAT GETS SAVED TO THE USER'S DATABASE ARE THE TOP 4 FIELDS IN THE SETTINGS TAB (FROM INCLUDES/OPTIONS.PHP - LICENCE KEY, NOTIFICATION EMAIL, SELECTED FORM BUILDER AND TIMEZONE.  EVERYTHING ELSE IS SIMPLY PUSHED OR PULLED FROM THE HIPAA FORMS SERVICE API.
 */

/*** ADMIN AJAX FUNCTION ***/
/* GET SUBMITTED FORMS */
function cm_hipaa_get_forms() {
    if (isset($_REQUEST)) {
        $firstName = sanitize_text_field($_REQUEST['first_name']);
        $lastName = sanitize_text_field($_REQUEST['last_name']);
        $phone = sanitize_text_field($_REQUEST['phone']);
        $email = sanitize_email($_REQUEST['email']);
        $limit = intval($_REQUEST['limit']);
        $page = intval($_REQUEST['page']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            // GET SUBMITTED FORMS FROM CODEMONKEYS HIPAA API
            $cmSubmittedForms = new cmHipaaForms;
            echo $cmSubmittedForms->getForms($firstName, $lastName, $phone, $email, $limit, $page, $nonce);
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_get_forms', 'cm_hipaa_get_forms' );
add_action( 'wp_ajax_nopriv_cm_hipaa_get_forms', 'cm_hipaa_get_forms' );

/* GENERATE PDF */
function cm_hipaa_generate_pdf() {
    if(isset($_REQUEST)) {
        $form_id = intval($_REQUEST['form_id']);
        $pdf_password = sanitize_text_field($_REQUEST['pdf_password']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            $hipaaForms = new cmHipaaForms;
            echo $hipaaForms->generatePdf($form_id, $pdf_password);
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_generate_pdf', 'cm_hipaa_generate_pdf' );
add_action( 'wp_ajax_nopriv_cm_hipaa_generate_pdf', 'cm_hipaa_generate_pdf' );

/* DELETE PDF - NOT NEEDED ANYMORE PROBABLY
function cm_hipaa_delete_pdf() {
    if(isset($_REQUEST)) {
        $url = esc_url($_REQUEST['url']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        $hipaaForms = new cmHipaaForms;
        echo $hipaaForms->deletePdf($url, $nonce);
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_delete_pdf', 'cm_hipaa_delete_pdf' );
add_action( 'wp_ajax_nopriv_cm_hipaa_delete_pdf', 'cm_hipaa_delete_pdf' );
*/

/* DELETE FORM */
function cm_hipaa_delete_form() {
    if(isset($_REQUEST)) {
        $licenseKey = esc_attr(get_option('license_key'));
        $form_id = intval($_REQUEST['form_id']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            $hipaaForms = new cmHipaaForms;
            echo $hipaaForms->deleteForm($licenseKey, $form_id);
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_delete_form', 'cm_hipaa_delete_form' );
add_action( 'wp_ajax_nopriv_cm_hipaa_delete_form', 'cm_hipaa_delete_form' );

/* GET ACCESS LOGS */
function cm_hipaa_get_logs() {
    if (isset($_REQUEST)) {
        $startDate = sanitize_text_field($_REQUEST['start_date']);
        $endDate = sanitize_text_field($_REQUEST['end_date']);
        $limit = intval($_REQUEST['limit']);
        $page = intval($_REQUEST['page']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            // GET LOGS FROM CODEMONKEYS HIPAA API
            $cmHipaaForms = new cmHipaaForms;
            echo $cmHipaaForms->getAccessLogs($startDate, $endDate, $limit, $page);
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_get_logs', 'cm_hipaa_get_logs' );
add_action( 'wp_ajax_nopriv_cm_hipaa_get_logs', 'cm_hipaa_get_logs' );

/* GET BAA FORM */
function cm_hipaa_get_baa_form() {
    if (isset($_REQUEST)) {
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            // GET BAA FROM CODEMONKEYS HIPAA API
            $cmHipaaForms = new cmHipaaForms;
            echo $cmHipaaForms->getBaaForm();
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_get_baa_form', 'cm_hipaa_get_baa_form' );
add_action( 'wp_ajax_nopriv_cm_hipaa_get_baa_form', 'cm_hipaa_get_baa_form' );

/* SUBMIT BAA FORM */
function cm_hipaa_submit_baa_form() {
    if (isset($_REQUEST)) {
        $form = esc_html($_REQUEST['form']);
        $signature = $_REQUEST['signature'];
        $signersName = sanitize_text_field($_REQUEST['signers_name']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            // SUBMIT BAA FORM TO CODEMONKEYS HIPAA API
            $cmHipaaForms = new cmHipaaForms;
            echo $cmHipaaForms->submitBaaForm($form, $signature, $signersName);
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_submit_baa_form', 'cm_hipaa_submit_baa_form' );
add_action( 'wp_ajax_nopriv_cm_hipaa_submit_baa_form', 'cm_hipaa_submit_baa_form' );

/* GET BAA PDF */
function cm_hipaa_get_baa_pdf() {
    if (isset($_REQUEST)) {
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            // GET BAA PDF URL FROM CODEMONKEYS HIPAA API
            $cmHipaaForms = new cmHipaaForms;
            echo $cmHipaaForms->getBaaPdf();
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_get_baa_pdf', 'cm_hipaa_get_baa_pdf' );
add_action( 'wp_ajax_nopriv_cm_hipaa_get_baa_pdf', 'cm_hipaa_get_baa_pdf' );

/* GET SUPPORT TICKETS */
function cm_hipaa_get_support_tickets() {
    if (isset($_REQUEST)) {
        $status = sanitize_text_field($_REQUEST['status']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            // GET SUPPORT TICKETS FROM CODEMONKEYS SUPPORT TICKETS API
            $cmHipaaForms = new cmHipaaForms;
            echo $cmHipaaForms->getSupportTickets($status);
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_get_support_tickets', 'cm_hipaa_get_support_tickets' );
add_action( 'wp_ajax_nopriv_cm_hipaa_get_support_tickets', 'cm_hipaa_get_support_tickets' );

/* SUBMIT SUPPORT TICKETS */
function cm_hipaa_submit_support_ticket() {
    if (isset($_REQUEST)) {
        $priority = sanitize_text_field($_REQUEST['priority']);
        $subject = sanitize_text_field($_REQUEST['subject']);
        $message = sanitize_textarea_field($_REQUEST['message']);
        $parentId = intval($_REQUEST['parent_id']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            // POST TICKET TO CODEMONKEYS SUPPORT TICKET API
            $cmHipaaForms = new cmHipaaForms;
            echo $cmHipaaForms->submitSupportTicket($priority, $subject, $message, $parentId);
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_submit_support_ticket', 'cm_hipaa_submit_support_ticket' );
add_action( 'wp_ajax_nopriv_cm_hipaa_submit_support_ticket', 'cm_hipaa_submit_support_ticket' );

/* CLOSE SUPPORT TICKETS */
function cm_hipaa_close_support_ticket() {
    if (isset($_REQUEST)) {
        $ticketId = intval($_REQUEST['ticket_id']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            // CLOSE TICKET IN CODEMONKEYS SUPPORT TICKET API
            $cmHipaaForms = new cmHipaaForms;
            echo $cmHipaaForms->closeSupportTicket($ticketId);
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_close_support_ticket', 'cm_hipaa_close_support_ticket' );
add_action( 'wp_ajax_nopriv_cm_hipaa_close_support_ticket', 'cm_hipaa_close_support_ticket' );


/*** FRONT END AJAX FUNCTION ***/
/* SUBMIT FORM */
function cm_hipaa_submit_hipaa_form() {
    if (isset($_REQUEST)) {
        $licenseKey = esc_attr(get_option('license_key'));
        $notificationEmail = esc_attr(get_option('notification_email'));
        $formId = sanitize_text_field($_REQUEST['formId']);
        $formFields = $_REQUEST['formFields']; // ARRAY OF ENTIRE FIELDS, SANITIZING THIS HERE HAS HIGH POTENTIAL TO BREAK BUT REMEMBER THIS IS NOT SAVED TO THEIR DATABASE RATHER IS ENCRYPTED AND SENT TO BE STORED THROUGH OUR API.  IT IS SANITIZED ONCE IT IS PULLED BACK FROM THE API AND DECRYPTED
        $signature = sanitize_text_field($_REQUEST['signature']);
        $nonce = sanitize_text_field($_REQUEST['nonce']);
        $firstName = '';
        $lastName = '';
        $email = '';
        $phone = '';
        $fields = array();

        if(!wp_verify_nonce($nonce, 'cm-hipaa-forms-nonce')) {
            $results = array(
                'error' => 'Nonce invalid'
            );

            echo json_encode($results);
        } else {
            // GET CALDERA FORM DATA
            $formData = Caldera_Forms_Forms::get_form($formId);
            $formName = $formData['name'];
            $fieldsData = $formData['fields'];

            foreach ($formFields as $key => $value) {
                $dataField = $value['data_field'];
                $fieldId = $value['field_id'];
                $fieldLabel = strip_tags($value['label']);
                $fieldValue = $value['value'];

                // SET THE FIELD ID TO THE DATA_FIELD VALUE, IF DOESN'T EXIST SET TO TO ACTUAL ID "id_1"
                if ($dataField) {
                    $fieldIdMain = $dataField;
                } else {
                    $fieldIdMain = $fieldId;
                }

                // GET CALDERA FIELD DATA FOR THIS FIELD
                $fieldData = $fieldsData[$fieldIdMain];

                // GET FIELD TYPE
                $fieldType = $fieldData['type'];

                // GET CALDERA FIELD SLUG
                $fieldSlug = $fieldData['slug'];

                // SET PRIMARY FIELDS
                if ($fieldSlug == 'first_name') {
                    $firstName = $fieldValue;
                } else {
                    if ($fieldSlug == 'last_name') {
                        $lastName = $fieldValue;
                    } else {
                        if ($fieldSlug == 'email') {
                            $email = $fieldValue;
                        } else {
                            if ($fieldSlug == 'phone') {
                                $phone = $fieldValue;
                            }
                        }
                    }
                }

                // PUSH FIELD TO ARRAY
                $fields[] = array(
                    'label' => $fieldLabel,
                    'value' => $fieldValue,
                    'type' => $fieldType
                );
            }

            $fields = json_encode($fields, JSON_UNESCAPED_SLASHES);

            // SUBMIT FORM TO CODEMONKEYS HIPAA API
            $cmSubmitForm = new cmHipaaForms;
            echo $cmSubmitForm->sendForm($licenseKey, $notificationEmail, $formName, $firstName, $lastName, $email, $phone, $fields, $signature);
        }
    }

    die();
}
add_action( 'wp_ajax_cm_hipaa_submit_hipaa_form', 'cm_hipaa_submit_hipaa_form' );
add_action( 'wp_ajax_nopriv_cm_hipaa_submit_hipaa_form', 'cm_hipaa_submit_hipaa_form' );