<?php
/**
 * Created by Code Monkeys LLC
 * http://www.codemonkeysllc.com
 * User: Spencer
 * Date: 10/24/2016
 * Time: 4:24 PM
 */

class cmHipaaForms {
    /*
     * ADMIN CLASSES
     */

    /*** ENCRYPT ***/
    public static function encrypt($string, $key, $iv) {
        $output = false;
        $encrypt_method = "AES-256-CBC";

        $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
        $output = base64_encode($output);

        return $output;
    }

    /*** DECRYPT ***/
    public static function decrypt($string, $key, $iv) {
        $encrypt_method = "AES-256-CBC";

        return openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
    }

    /*** GENERATE RANDOM STRING ***/
    public static function randStringGen($length) {
        $chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()_-=+;:,.?";

        return substr( str_shuffle( $chars ), 0, $length );
    }

    /*** GENERATE ENCRYPTION KEY ***/
    public static function keygen() {
        $secret_key = self::randStringGen(8);

        return hash('sha256', $secret_key);
    }

    /*** GENERATE IV KEY ***/
    public static function ivgen() {
        $secret_iv = self::randStringGen(8);

        // iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
        return substr(hash('sha256', $secret_iv), 0, 16);
    }

    public static function getRootDomain($tld = false) {
        $url = get_site_url();
        $pieces = parse_url($url);
        $domain = isset($pieces['host']) ? $pieces['host'] : '';
        if (preg_match('/(?P<domain>[a-z0-9][a-z0-9\-]{1,63}\.[a-z\.]{2,6})$/i', $domain, $m)) {
            return ($tld === true) ? substr($m['domain'], ($pos = strpos($m['domain'], '.')) !== false ? $pos + 1 : 0) : $m['domain'];
        }
        return false;
    }

    private function getAccount($licenseKey, $domain) {
        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'getaccount',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain()
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        foreach($output as $data) {
            $error = $data->error;

            if($error) {
                $this->error = $error;
            } else {
                $success = $data->success;

                if ($success == 'success') {
                    $this->id = $data->id;
                    $this->customer_id = $data->customer_id;
                    $this->partner_id = $data->partner_id;
                    $this->first_name = $data->first_name;
                    $this->last_name = $data->last_name;
                    $this->email = $data->email;
                    $this->expiration_date = $data->expiration_date;
                    $this->baa = $data->baa;
                } else {
                    $this->error = 'API did not return a success message';
                }
            }
        }

        return $this;
    }

    /**** GET SUBMITTED FORMS ***/
    public function getForms($firstName, $lastName, $phone, $email, $limit, $page) {
        $licenseKey = esc_attr(get_option('license_key'));
        $timeZone = esc_attr(get_option('time_zone'));
        if ($timeZone == 'central') {
            $tz = 'America/Chicago';
        } else {
            if ($timeZone == 'eastern') {
                $tz = 'America/New_York';
            } else {
                if ($timeZone == 'mountain') {
                    $tz = 'America/Denver';
                } else {
                    if ($timeZone == 'pacific') {
                        $tz = 'America/Los_Angeles';
                    }
                }
            }
        }

        // SET OFFSET
        $offset = $page * $limit;

        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'get',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain(),
            'first_name' => $firstName,
            'last_name' => $lastName,
            'phone' => $phone,
            'email' => $email,
            'limit' => $limit,
            'offset' => $offset
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        if ($output) {
            foreach ($output as $data) {
                $error = $data->error;

                if ($error) {
                    if ($error == 'No BAA') {
                        $results = array(
                            'error' => $error,
                            'content' => '<div class="cm-sign-baa-button">You must sign the BAA Agreement!</div>'
                        );
                    } else {
                        $results = array(
                            'error' => $error,
                            'content' => $error
                        );
                    }
                } else {
                    $success = $data->success;

                    if ($success == 'success') {
                        $formsData = $data->forms;
                        $totalResults = $data->total_results;

                        $forms = array();
                        foreach ($formsData as $formData) {
                            $formId = $formData->form_id;
                            $formName = $formData->form_name;
                            $formFirstName = $formData->first_name;
                            $formLastName = $formData->last_name;
                            $formEmail = $formData->email;
                            $formPhone = $formData->phone;
                            $usersTimezone = new DateTimeZone($tz);
                            $dtObj = new DateTime($formData->timestamp);
                            $dtObj->setTimeZone($usersTimezone);
                            $formDate = $dtObj->format('m-d-Y g:i a');
                            $tzAbbr = $dtObj->format('T');
                            $formFields = $formData->fields;
                            $encryptKey = $formData->encrypt_key;
                            $encryptIv = $formData->encrypt_iv;
                            $status = $formData->status;
                            $signature = $formData->signature;

                            // ONLY GET NON-DELETED FORMS (STATUS = 1)
                            if ($status == 1) {
                                // SET SIGNATURE
                                if ($signature) {
                                    $signature = '
                                        <div class="cm-hipaa-submitted-form-signature">
                                            <img src="' . $signature . '" alt="Signature" />
                                        </div>
                                    ';
                                }

                                // DECRYPT FIELDS
                                $decryptedFields = json_decode(self::decrypt($formFields, $encryptKey, $encryptIv));

                                // LOOP FIELDS AND PUSH LABEL - VALUE TO ARRAY
                                $fields = array();
                                $i = 1;
                                foreach ($decryptedFields as $decryptedField) {
                                    $fieldType = $decryptedField->type;

                                    if ($fieldType == 'textarea' || $fieldType == 'paragraph') {
                                        $fields[] = '
                                            <div class="clearfix"></div>
                                            <div class="cm-hipaa-submitted-form-field ' . $fieldType . ' full-width">
                                                <div class="cm-hipaa-submitted-form-field-inner">
                                                    <div class="cm-hipaa-submitted-form-field-label">
                                                        ' . str_replace('*', '', stripslashes($decryptedField->label)) . '
                                                    </div>
                                                    <div class="cm-hipaa-submitted-form-field-value">
                                                        ' . stripslashes($decryptedField->value) . '
                                                    </div>
                                                </div>
                                            </div>
                                        ';
                                    } else {
                                        $fields[] = '
                                            <div class="cm-hipaa-submitted-form-field ' . $fieldType . ' float-left quarter-width">
                                                <div class="cm-hipaa-submitted-form-field-inner">
                                                    <span class="cm-hipaa-submitted-form-field-label">
                                                        ' . str_replace('*', '', stripslashes($decryptedField->label)) . ':
                                                    </span>
                                                    <span class="cm-hipaa-submitted-form-field-value">
                                                        ' . stripslashes($decryptedField->value) . '
                                                    </span>
                                                </div>
                                            </div>
                                        ';
                                    }


                                    if ($i % 4 == 0) {
                                        $fields[] = '
                                            <div class="clearfix"></div>
                                        ';
                                    }

                                    if ($fieldType == 'textarea') {
                                        $i = 1;
                                    } else {
                                        $i++;
                                    }
                                }

                                $forms[] = '
                                    <tr id="cm-hipaa-form-id-' . $formId . '" class="cm-hipaa-submitted-form" data-id="' . $formId . '">
                                        <td>
                                            <div class="cm-hipaa-submitted-form-options">
                                                <div class="cm-hipaa-submitted-form-toggle-fields">
                                                    <i class="far fa-caret-square-down"></i>
                                                </div>
                                                <div class="cm-hipaa-submitted-form-delete">
                                                    <i class="fas fa-trash-alt"></i>
                                                </div>
                                            </div>
                                        </td>
                                        <td>
                                            ' . $formFirstName . '
                                        </td>
                                        <td>
                                            ' . $formLastName . '
                                        </td>
                                        <td>
                                            ' . $formName . '
                                        </td>
                                        <td>
                                            ' . $formDate . ' ' . $tzAbbr . '
                                        </td>
                                        <td>
                                            <div class="cm-hipaa-submitted-form-pdf-section">
                                                <div class="cm-hipaa-generate-pdf-modal-button cm-ghost-button" data="' . $formId . '">
                                                    Generate PDF
                                                </div>
                                            </div>
                                        </td>
                                    </tr>
                                    <tr class="cm-hipaa-submitted-form-fields-row">
                                        <td colspan="7">
                                            <div class="cm-hipaa-submitted-form-fields">
                                                <div class="cm-hipaa-submitted-form-fields-inner">
                                                    <h2>' . $formName . '</h2>
                                                    ' . implode('', $fields) . '
                                                    <div class="clearfix"></div>
                                                    ' . $signature . '
                                                </div>
                                            </div>
                                        </td>
                                    </tr>
                                ';
                            }
                        }

                        $content = '
                            <table class="cm-hipaa-submitted-forms">
                                <thead class="cm-table-header">
                                    <td>
                                    </td>
                                    <td>
                                        FIRST NAME
                                    </td>
                                    <td>
                                        LAST NAME
                                    </td>
                                    <td>
                                        FORM NAME
                                    </td>
                                    <td>
                                        SUBMISSION DATE
                                    </td>
                                    <td>
                                        PDF
                                    </td>
                                </thead>
                                ' . implode('', $forms) . '
                            </table>
                        ';

                        $results = array(
                            'success' => 'success',
                            'content' => $content,
                            'total_results' => $totalResults
                        );
                    } else {
                        $results = array(
                            'error' => 'API did not return a success message',
                            'content' => 'API did not return a success message'
                        );
                    }
                }
            }
        } else {
            $results = array(
                'error' => 'No response from API',
                'content' => 'No response from API'
            );
        }

        return json_encode($results);
    }

    /*** GET CALDERA FORMS LIST ***/
    public function getCalderaForms($enabledForms) {
        /*** GET CALDERA FORMS (ID & FORM NAME) ***/
        $forms = Caldera_Forms_Forms::get_forms(true, true);

        if (!empty($forms)) {
            $formsArray = array();
            foreach ($forms as $form) {
                $errors = array();
                $errorClass = '';
                $errorHeading = '';
                $errorIcon = '';
                $fieldsArray = array();
                $fieldSlugs = array();
                $formId = $form['ID'];
                $formName = $form['name'];
                $formData = Caldera_Forms_Forms::get_form($formId);
                $fields = $formData['fields'];

                /*** GET FIELDS ***/
                foreach ($fields as $field) {
                    $fieldSlugs[] = $field['slug'];
                    $fieldsArray[] = $field['label'] . ' - ' . $field['slug'];
                }

                /*** SET FIELD ERRORS ***/
                if (!in_array('first_name', $fieldSlugs)) {
                    $errors[] = 'No field with first_name slug';
                }

                if (!in_array('last_name', $fieldSlugs)) {
                    $errors[] = 'No field with last_name slug';
                }

                if (!in_array('email', $fieldSlugs)) {
                    $errors[] = 'No field with email slug';
                }

                $fieldsArray = implode('<br />', $fieldsArray);
// TODO: UPDATE FONT AWESOME ICONS FOR V5
                if (!empty($errors)) {
                    $errorClass = ' errors';
                    $errorIcon = '
                        <span class="cm-hipaa-toggle-errors"> <i class="fas fa-exclamation-triangle"></i></span>
                    ';
                    $errorHeading = '<h4>Errors</h4>';
                }

                $errors = implode('<br />', $errors);

                if(in_array($formId, $enabledForms)) {
                    $formSelectIcon = '<i class="far fa-check-square"></i>';
                    $formSelected = ' selected';
                } else {
                    $formSelectIcon = '<i class="far fa-square"></i>';
                    $formSelected = '';
                }

                $formsArray[] = '
                    <div id="' . $formId . '" class="cm-hipaa-select-form-item' . $errorClass . $formSelected . '">
                        <div class="cm-hipaa-select-form-item-name">
                            <span class="cm-hipaa-form-select">
                                ' . $formSelectIcon . '
                            </span>
                            <span class="cm-hipaa-toggle-fields" title="Show Fields">
                                <i class="far fa-caret-square-down"></i>
                            </span>
                            ' . $errorIcon . ' ' . $formName . ' - Caldera Form ID: ' . $formId . '
                        </div>
                        <div class="cm-hipaa-select-form-item-fields">
                            <h4>Fields:</h4>
                            ' . $fieldsArray . '
                        </div>
                        <div class="cm-hipaa-select-form-item-errors">
                            ' . $errorHeading . '
                            ' . $errors . '
                        </div>
                    </div>
                ';
            }

            return implode('', $formsArray);
        }

        return false;
    }

    /*** GENERATE PDF ***/
    public function generatePdf($formId, $pdfPassword) {
        $licenseKey = esc_attr(get_option('license_key'));
        $timeZone = esc_attr(get_option('time_zone'));

        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'getpdf',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain(),
            'form_id' => $formId,
            'time_zone' => $timeZone,
            'pdf_password' => $pdfPassword
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        if($output) {
            foreach($output as $data) {
                $error = $data->error;

                if ($error) {
                    return $error;
                } else {
                    $success = $data->success;

                    if($success == 'success') {
                        $pdf_url = $data->pdf_url;
                        $pdf_password = $data->pdf_password;

                        $modal_message = '
                            <div class="cm-hipaa-forms-generate-pdf-success">
                                <div class="cm-hipaa-forms-generate-pdf-success">
                                    <p>
                                        The PDF has been generated with encryption and password protection.
                                    </p>
                                    <p>
                                        Click the link below to open the PDF and enter your password (' . $pdf_password . ').
                                    </p>
                                    <p>
                                        After entering the password you will be able to view, print and download the form.
                                    </p>
                                    <p>
                                        DO NOT LOSE YOUR PASSWORD!
                                    </p>
                                    <p>
                                        You will not be able to open the PDF without the password even after it is downloaded.
                                    </p>
                                </div>
                                <a class="cm-button" href="' . $pdf_url . '" target="_blank"><i class="fa fa-file-pdf-o" aria-hidden="true"></i> OPEN PDF</a>
                            </div>
                        ';

                        $form_pdf = '
                            <div class="cm-hipaa-submitted-form-pdf-url">
                                <a href="' . $pdf_url . '" target="_blank"><i class="fa fa-file-pdf-o" aria-hidden="true"></i> PDF</a>
                            </div>
                        ';

                        $form_password = '
                            <div class="cm-hipaa-submitted-form-pdf-password">
                                <i class="fa fa-key" aria-hidden="true"></i> ' . $pdf_password . '
                            </div>
                        ';

                        $results = array(
                            'pdf_url' => $pdf_url,
                            'pdf_password' => $pdf_password,
                            'form_pdf' => $form_pdf,
                            'form_password' => $form_password,
                            'modal_message' => $modal_message
                        );

                        return json_encode($results);
                    } else {
                        return 'API did not return a success message: ' . $success;
                    }
                }
            }
        } else {
            return 'There was an error in the api';
        }

        return false;
    }

    /*** DELETE PDF (NOT NEEDED ANYMORE)
    public function deletePdf($url) {
        $licenseKey = esc_attr(get_option('license_key'));

        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'deletepdf',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain(),
            'pdf_url' => $url
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        if ($output) {
            foreach ($output as $data) {
                $error = $data->error;

                if ($error) {
                    $results = array(
                        'error' => $error
                    );
                } else {
                    $success = $data->success;

                    if($success == 'success') {
                        $results = array(
                            'content' => 'PDF Deleted'
                        );
                    }
                }
            }
        } else {
            $results = array(
                'content' => 'No response from API'
            );
        }

        return json_encode($results);
    }
     * /

    /*** DELETE FORM ***/
    public function deleteForm($licenseKey, $formId) {
        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'delete',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain(),
            'form_id' => $formId
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        foreach($output as $data) {
            $error = $data->error;

            if($error) {
                $results = array(
                    'error' => $error
                );
            } else {
                $results = array(
                    'success' => $data->success
                );
            }
        }

        return json_encode($results);
    }

    /*** ACCESS LOG ***/
    public function accessLog($userId, $userName, $firstName, $lastName, $email, $userRole, $status) {
        $licenseKey = esc_attr(get_option('license_key'));
        $userRole = implode(',', $userRole);

        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'log',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain(),
            'user_id' => $userId,
            'user_name' => $userName,
            'first_name' => $firstName,
            'last_name' => $lastName,
            'email' => $email,
            'user_role' => $userRole,
            'status' => $status
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        foreach($output as $data) {
            $error = $data->error;

            if($error) {
                return $error;
            }
        }

        return false;
    }

    /*** GET ACCESS LOG ***/
    public function getAccessLogs($startDate, $endDate, $limit, $page) {
        $licenseKey = esc_attr(get_option('license_key'));
        $timeZone = esc_attr(get_option('time_zone'));
        if($timeZone == 'central') {
            $tz = 'America/Chicago';
        } else if($timeZone == 'eastern') {
            $tz = 'America/New_York';
        } else if($timeZone == 'mountain') {
            $tz = 'America/Denver';
        } else if($timeZone == 'pacific') {
            $tz = 'America/Los_Angeles';
        }

        // SET OFFSET
        $offset = $page * $limit;

        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'getlogs',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain(),
            'start_date' => $startDate,
            'end_date' => $endDate,
            'limit' => $limit,
            'offset' => $offset
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        foreach($output as $data) {
            $error = $data->error;

            if($error) {
                $results = array(
                    'error' => $error,
                    'content' => $error
                );
            } else {
                $success = $data->success;
                $totalResults = $data->total_results;

                if ($success == 'success') {
                    $logsData = $data->logs;

                    $logs = array();
                    foreach ($logsData as $logData) {
                        $logUserId = $logData->user_id;
                        $logUserName = $logData->username;
                        $logFirstName = $logData->first_name;
                        $logLastName = $logData->last_name;
                        $logEmail = $logData->email;
                        $logUserRoles = $logData->user_role;
                        $logStatus = $logData->status;
                        $domain = $logData->domain;
                        $usersTimezone = new DateTimeZone($tz);
                        $dtObj = new DateTime($logData->timestamp);
                        $dtObj->setTimeZone($usersTimezone);
                        $logDate = $dtObj->format('m-d-Y g:i a');
                        $tzAbbr = $dtObj->format('T');

                        if($logStatus == 1) {
                            $logStatus = 'Allowed';
                        } else if($logStatus == 0) {
                            $logStatus = 'Denied';
                        }

                        $logs[] = '
                            <tr class="cm-hipaa-forms-log">
                                <td>
                                    ' . $logUserId . '
                                </td>
                                <td>
                                    ' . $logUserName . '
                                </td>
                                <td>
                                    ' . $logFirstName . ' ' . $logLastName . '
                                </td>
                                <td>
                                    ' . $logEmail . '
                                </td>
                                <td>
                                    ' . $logUserRoles . '
                                </td>
                                <td>
                                    ' . $logStatus . '
                                </td>
                                <td>
                                    ' . $domain . '
                                </td>
                                <td>
                                    ' . $logDate . ' ' . $tzAbbr . '
                                </td>
                            </tr>
                        ';
                    }

                    $content = '
                            <table class="cm-hipaa-forms-logs">
                                <thead class="cm-table-header">
                                    <td>
                                        USER ID
                                    </td>
                                    <td>
                                        USERNAME
                                    </td>
                                    <td>
                                        NAME
                                    </td>
                                    <td>
                                        EMAIL
                                    </td>
                                    <td>
                                        USER ROLE
                                    </td>
                                    <td>
                                        STATUS
                                    </td>
                                    <td>
                                        DOMAIN
                                    </td>
                                    <td>
                                        DATE/TIME
                                    </td>
                                </thead>
                                ' . implode('', $logs) . '
                            </table>
                        ';

                    $results = array(
                        'success' => 'success',
                        'content' => $content,
                        'total_results' => $totalResults
                    );
                } else {
                    $results = array(
                        'error' => 'API did not return a success message',
                        'content' => 'API did not return a success message'
                    );
                }
            }
        }

        return json_encode($results);
    }

    /*** GET BAA FORM ***/
    public function getBaaForm() {
        $licenseKey = esc_attr(get_option('license_key'));

        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'getbaaform',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain()
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        foreach($output as $data) {
            $error = $data->error;

            if ($error) {
                $results = array(
                    'error' => $error
                );
            } else {
                $success = $data->success;

                if($success == 'success') {
                    if($data->form == 'No Baa') {
                        $results = array(
                            'success' => $data->success,
                            'form' => '<div class="cm-sign-baa-button">You must sign the BAA Agreement!</div>'
                        );
                    } else {
                        $results = array(
                            'success' => $data->success,
                            'form' => $data->form
                        );
                    }
                } else {
                    $results = array(
                        'error' => $error
                    );
                }
            }
        }

        return json_encode($results);
    }

    /*** SUBMIT BAA FORM ***/
    public function submitBaaForm($form, $signature, $signersName) {
        $licenseKey = esc_attr(get_option('license_key'));
        $timezone = esc_attr(get_option('time_zone'));

        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'submitbaaform',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain(),
            'timezone' => $timezone,
            'form' => $form,
            'signature' => $signature,
            'signers_name' => $signersName
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        foreach($output as $data) {
            $error = $data->error;

            if ($error) {
                return $error;
            } else {
                $success = $data->success;

                if($success == 'success') {
                    return $data->form;
                } else {
                    return $error;
                }
            }
        }

        return false;
    }

    /*** GET BAA PDF FORM URL ***/
    public function getBaaPdf() {
        $licenseKey = esc_attr(get_option('license_key'));

        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'getbaapdf',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain()
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        foreach($output as $data) {
            $error = $data->error;

            if ($error) {
                $results = array(
                    'error' => $error,
                    'content' => 'No open tickets'
                );
            } else {
                $success = $data->success;

                if($success == 'success') {
                    $results = array(
                        'success' => 'success',
                        'content' => $data->form
                    );
                } else {
                    $results = array(
                        'error' => $error,
                        'content' => $error
                    );
                }
            }
        }

        return json_encode($results);
    }

    /*** GET SUPPORT TICKETS ***/
    public function getSupportTickets($status) {
        $licenseKey = esc_attr(get_option('license_key'));
        $domain = self::getRootDomain();

        // GET ACCOUNT DATA
        $account = self::getAccount($licenseKey, $domain);

        // Create curl resource
        $curl_url = 'https://stagingserver.online/codemonkeysdev/support-api/';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'get',
            'account_id' => $account->id,
            'license_key' => $licenseKey,
            'domain' => $domain,
            'status' => $status
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        if($output) {
            foreach ($output as $data) {
                $error = $data->error;

                if ($error) {
                    $results = array(
                        'error' => $error
                    );
                } else {
                    $success = $data->success;

                    if($success == 'success') {
                        if($data->tickets) {
                            $timeZone = esc_attr(get_option('time_zone'));
                            if($timeZone == 'central') {
                                $tz = 'America/Chicago';
                            } else if($timeZone == 'eastern') {
                                $tz = 'America/New_York';
                            } else if($timeZone == 'mountain') {
                                $tz = 'America/Denver';
                            } else if($timeZone == 'pacific') {
                                $tz = 'America/Los_Angeles';
                            }

                            $ticketsArr = array();
                            foreach ($data->tickets as $ticket) {
                                if(!$ticket->parent_id) {
                                    // SET DATE/TIME BASED ON USER'S TIMEZONE
                                    $usersTimezone = new DateTimeZone($tz);
                                    $dtObj = new DateTime($ticket->timestamp);
                                    $dtObj->setTimeZone($usersTimezone);
                                    $ticketDate = $dtObj->format('M jS Y g:i a');
                                    $tzAbbr = $dtObj->format('T');

                                    // SET PRIORITY
                                    if ($ticket->priority == 'high') {
                                        $priorityClass = 'cm-support-ticket-priority-high';
                                        $priorityTitle = 'High Priority';
                                    } else {
                                        if ($ticket->priority == 'medium') {
                                            $priorityClass = 'cm-support-ticket-priority-medium';
                                            $priorityTitle = 'Medium Priority';
                                        } else {
                                            if ($ticket->priority == 'low') {
                                                $priorityClass = 'cm-support-ticket-priority-low';
                                                $priorityTitle = 'Low Priority';
                                            }
                                        }
                                    }

                                    // GET REPLIES TO THIS TICKET
                                    $replies = array();
                                    foreach ($data->tickets as $reply) {
                                        if($reply->parent_id == $ticket->id) {
                                            // SET DATE/TIME BASED ON USER'S TIMEZONE
                                            $replyTimezone = new DateTimeZone($tz);
                                            $replyDtObj = new DateTime($reply->timestamp);
                                            $replyDtObj->setTimeZone($replyTimezone);
                                            $replyDate = $replyDtObj->format('M jS Y g:i a');
                                            $replyTzAbbr = $replyDtObj->format('T');

                                            $replies[] = '
                                                <div class="cm-support-ticket-reply">
                                                    <div class="cm-support-ticket-reply-datetime">
                                                        ' . $replyDate . ' ' . $replyTzAbbr . '
                                                    </div>
                                                    <div class="cm-support-ticket-reply-message">
                                                        ' . stripslashes($reply->message) . '
                                                    </div>
                                                    <div class="cm-support-ticket-reply-submitter">
                                                        Reply from: ' . $reply->first_name . ' ' . $reply->last_name . '
                                                    </div>
                                                </div>
                                            ';
                                        }
                                    }

                                    // SET SUBMIT AND CLOSE BUTTONS BASED ON STATUS OF TICKET
                                    if($ticket->status == 'open') {
                                        $closeTicketButton = '
                                            <div class="cm-support-ticket-close cm-button" data="' . $ticket->id . '">
                                                CLOSE TICKET
                                            </div>
                                        ';

                                        $submitButton = '
                                            <div class="cm-support-ticket-reply-submit cm-button" data="' . $ticket->id . '">
                                                SUBMIT
                                            </div>
                                        ';
                                    } else if($ticket->status == 'closed') {
                                        $submitButton = '
                                            <div class="cm-support-ticket-reply-submit cm-button" data="' . $ticket->id . '">
                                                REOPEN TICKET
                                            </div>
                                        ';
                                    }

                                    // BUILD THE TICKET
                                    $ticketsArr[] = '
                                        <div class="cm-support-ticket-wrapper">
                                            <div class="cm-support-ticket">
                                                <div class="cm-support-ticket-heading">
                                                    <div class="cm-support-ticket-datetime">
                                                        <span class="cm-support-ticket-toggle"><i class="far fa-caret-square-down"></i></span> <span class="' . $priorityClass . '" title="' . $priorityTitle . '"><i class="fas fa-flag"></i></span> ' . $ticketDate . ' ' . $tzAbbr . '
                                                    </div>
                                                    <div class="cm-support-ticket-subject">
                                                        ' . stripslashes($ticket->subject) . '
                                                    </div>
                                                    <div class="cm-support-ticket-submitter">
                                                        Submitted by: ' . $ticket->first_name . ' ' . $ticket->last_name . '
                                                    </div>
                                                </div>
                                                <div class="cm-support-ticket-body">
                                                    <div class="cm-support-ticket-message">
                                                        ' . stripslashes($ticket->message) . '
                                                    </div>
                                                    <div class="cm-support-ticket-replies-wrapper">
                                                        ' . implode('', $replies) . '
                                                    </div>
                                                    <div class="cm-support-ticket-reply-wrapper">
                                                        <div class="cm-support-ticket-reply-input-wrapper">
                                                            <textarea class="cm-support-ticket-reply-input" placeholder="Reply..."></textarea>
                                                        </div>
                                                        <div class="cm-support-ticket-reply-buttons-wrapper">
                                                            ' . $submitButton . '
                                                            ' . $closeTicketButton . '
                                                            <div class="clearfix"></div>
                                                            <div class="cm-support-ticket-reply-notice"></div>
                                                        </div>
                                                    </div>
                                                </div>
                                            </div>
                                        </div>
                                    ';
                                }
                            }

                            $tickets = '
                                <div class="cm-tickets-wrapper">
                                    <div class="cm-support-tickets">
                                        ' . implode('', $ticketsArr) . '
                                    </div>
                                </div>
                            ';

                            $results = array(
                                'success' => 'success',
                                'tickets' => $tickets
                            );
                        } else {
                            $results = array(
                                'success' => 'success',
                                'tickets' => 'No open tickets'
                            );
                        }
                    } else {
                        $results = array(
                            'error' => 'Support API did not return a success message.'
                        );
                    }
                }
            }
        }

        return json_encode($results);
    }

    /*** SUBMIT SUPPORT TICKET ***/
    public function submitSupportTicket($priority, $subject, $message, $parent_id) {
        $licenseKey = esc_attr(get_option('license_key'));
        $domain = self::getRootDomain();
        $product = 'HIPAA FORMS WP Plugin';

        // GET USER ROLES AND SET APPROVED/NOT APPROVED
        $user = wp_get_current_user();

        $user_id = $user->ID;
        $user_name = $user->user_login;
        $user_display_name = $user->display_name;
        $name_pieces = explode(" ", $user_display_name);
        $user_first_name = $name_pieces[0];
        $user_last_name = $name_pieces[1];
        $user_email = $user->user_email;
        $user_roles = $user->roles;

        // GET ACCOUNT DATA
        $account = self::getAccount($licenseKey, $domain);

        // Create curl resource
        $curl_url = 'https://stagingserver.online/codemonkeysdev/support-api/';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'post',
            'parent_id' => $parent_id,
            'license_key' => $licenseKey,
            'account_id' => $account->id,
            'customer_id' => $user_id,
            'partner_id' => $account->partner_id,
            'domain' => $domain,
            'url' => get_site_url(),
            'first_name' => $user_first_name,
            'last_name' => $user_last_name,
            'email' => $user_email,
            'phone' => $account->phone,
            'products' => $product,
            'type' => 'product',
            'priority' => $priority,
            'subject' => stripslashes($subject),
            'message' => stripslashes($message),
            'status' => 'open'
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        if($output) {
            foreach ($output as $data) {
                $error = $data->error;

                if ($error) {
                    $results = array(
                        'error' => $error
                    );
                } else {
                    $success = $data->success;

                    if ($success == 'success') {
                        $results = array(
                            'success' => $data->success
                        );
                    } else {
                        $results = array(
                            'error' => $error
                        );
                    }
                }
            }
        }

        return json_encode($results);
    }

    /*** CLOSE SUPPORT TICKET ***/
    public function closeSupportTicket($ticket_id) {
        // Create curl resource
        $curl_url = 'https://stagingserver.online/codemonkeysdev/support-api/';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'closeticket',
            'ticket_id' => $ticket_id
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        if($output) {
            foreach ($output as $data) {
                $error = $data->error;

                if ($error) {
                    return $error;
                } else {
                    $success = $data->success;

                    if ($success == 'success') {
                        return $success;
                    } else {
                        return $error;
                    }
                }
            }
        }

        return false;
    }


    /*
     * FRONT END CLASSES
     */

    /*** SUBMIT FORM ***/
    public function sendForm($licenseKey, $notificationEmail, $formName, $firstName, $lastName, $email, $phone, $fields, $signature) {
        // ENCRYPT THE FORM FIELDS
        $key = self::keygen();
        $iv = self::ivgen();
        $fields = self::encrypt($fields, $key, $iv);

        // Create curl resource
        $curl_url = 'https://www.hipaaforms.online/hipaa-api';
        $curl = curl_init($curl_url);

        // Create post data
        $curl_post_data = array(
            'action' => 'post',
            'license_key' => $licenseKey,
            'domain' => self::getRootDomain(),
            'notification_email' => $notificationEmail,
            'form_name' => $formName,
            'first_name' => $firstName,
            'last_name' => $lastName,
            'email' => $email,
            'phone' => $phone,
            'fields' => $fields,
            'key' => $key,
            'iv' => $iv,
            'signature' => $signature
        );

        // Assign curl settings
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($curl_post_data));
        curl_setopt($curl, CURLOPT_REFERER, get_site_url());

        $output = json_decode(curl_exec($curl));

        // Close curl resource to free up system resources
        curl_close($curl);

        foreach($output as $data) {
            $error = $data->error;

            if($error) {
                return $error;
            } else {
                // SET FORM SUBMISSION NOTICE EMAIL
                $subject = 'HIPAA Form Submission';
                $body = '
                    <table width="600px" style="border-collapse:collapse;">
                        <tbody>
                            <tr>
                                <th><b>' . $formName . ' HIPAA Form Submission</b></th>
                            </tr>
                            <tr>
                                <td style="height:30px"></td>
                            </tr>
                            <tr>
                                <td>
                                    First Name: ' . $firstName . '
                                </td>
                            </tr>
                            <tr>
                                <td style="height:15px"></td>
                            </tr>
                            <tr>
                                <td>
                                    Last Name: ' . $lastName . '
                                </td>
                            </tr>
                            <tr>
                                <td style="height:15px"></td>
                            </tr>
                            <tr>
                                <td>
                                    Email: ' . $email . '
                                </td>
                            </tr>
                            <tr>
                                <td style="height:15px"></td>
                            </tr>
                            <tr>
                                <td>
                                    Phone: ' . $phone . '
                                </td>
                            </tr>
                            <tr>
                                <td style="height:30px"></td>
                            </tr>
                            <tr>
                                <td>
                                    Please log into the admin dashboard to view the form.
                                </td>
                            </tr>
                        </tbody>
                    </table>
                ';

                $url = $_SERVER['SERVER_NAME'];
                $fromMail = 'hipaaforms@' . $url;
                $headers = array();
                $headers[] = 'Content-Type: text/html; charset=UTF-8';
                $headers[] = 'From: HIPAA FORM SUBMISSION <' . $fromMail . '>';

                // SEND EMAIL NOTICE
                wp_mail( $notificationEmail, $subject, $body, $headers );

                // RETURN SUCCESS NOTICE
                return $data->success;
            }
        }

        return false;
    }
}